;; ============================================================================
;; file:	test_midi_in.asm          (Philips Music Module MIDI Framework)
;; version:     0.7a (March 1, 2008)
;;
;; description: This is an example usage of the msx_midi Music Module MIDI
;;    	        framework. This application will play notes on the PSG
;;              received on MIDI channel 1.
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the Lesser GNU General Public License as published by
;;  the Free Software Foundation; either version 2, or (at your option)
;;  any later version. See COPYING for more details.
;;
;; (C)2007,2008 by Vincent van Dam (vincentd@erg.verweg.com)
;; ============================================================================

	db   $fe
	dw   bin_begin
	dw   bin_end
	dw   bin_start

	org  $9000

bin_begin:
	;; include the midi framework
	include "msx_midi.asm"

bin_start:
	;; --------------------------------------------------------------------
	;; initialise the framework
	;; --------------------------------------------------------------------

	xor   a		; use internal clock (vblank) as midi clock
	call  msx_midi_init

	;; add a handler for all note on events received on midi channel 1
	ld    hl,noteon
	ld    e,1	; midi channel (0 to responds on all channels)
	call  msx_midi_add_noteon_handler

	;; add a handler for all note off events received on midi channel 1
	ld    hl,noteoff
	ld    e,1	; midi channel (0 to responds on all channels)
	call  msx_midi_add_noteoff_handler

	;; add a handler for the midi clock (=clocked interrupt handler)
	ld    hl,play
	call  msx_midi_add_clock_handler
	
	;; --------------------------------------------------------------------
	;; main loop, wait for space key to exit program.
	;; --------------------------------------------------------------------
_wait:
	ei	; make sure interrupts are enabled to receive midi data
	
	;; msx_midi_loop needs to be included in your main application loop and
	;; needs to be called as much as possible. it handles the internal
	;; clock and handles the dispatching of midi messages.
	call  msx_midi_loop

	;; wait for space key
        in    a,($aa)
        and   240
        or    8
        out   ($aa),a
        in    a,($a9)
        and   1
        jr    nz,_wait

	;; --------------------------------------------------------------------
	;; deinitialize midi framework (restore interrupt handlers)
	;; --------------------------------------------------------------------

	call  msx_midi_exit
	ret
	
	;; ====================================================================
	;; handler methods. these methods are called whenever a specific midi
	;; message was received on the midi channel the handler was configured
	;; to listen to. register [a] will always contain the midi channel.
	;; register [b] and register [c] will always contain extra information.
	;; the information differs per message, see msx_midi.asm for more info.
	;; ====================================================================
	
	;; note on handler
	
noteon:
	ld    a,(note_channel)
	ld    e,a
	ld    d,0
	ld    hl,note_buffer
	add   hl,de
	ld    (hl),c		; [c] contains note
	inc   a
	cp    3
	jr    c,noteon0
	xor   a
noteon0:	
	ld    (note_channel),a
	ret

	;; note off handler
	
noteoff:
	ld    hl,note_buffer+2
	ld    b,3
noteoff0:
	ld    a,(hl)
	cp    c			; [c] contains note
	jr    nz,noteoff1
	ld    a,-1
	ld    (hl),a
	ld    hl,note_channel
	dec   b
	ld    (hl),b
	ret
noteoff1:
	dec   hl
	djnz  noteoff0
	ret

	;; midi clock handler - play note

play:
	ld    hl,note_buffer+2
	ld    b,3
play0:
	;; get note for channel
	ld    a,(hl)
	dec   hl
	cp    -1 ; no note?
	jr    nz,play1
	;; mute channel
	ld    a,7
	add   b
	out   ($a0),a
	xor   a
	out   ($a1),a
	djnz  play0
	ret
play1:
	push  bc
	push  hl
	dec   b
	call  play_note
	pop   hl
	pop   bc
	ld    a,7
	add   b
	out   ($a0),a
	ld    a,15
	out   ($a1),a	
	djnz  play0
	ret
	
note_buffer:	db -1,-1,-1
note_channel:	db 0
	
	;; --------------------------------------------------------------------
	;; play a note on the psg
	;; --------------------------------------------------------------------
	;; in:	[a ] = note (0-95)
	;;      [b ] = channel (0-2)
	;; out:	-
	;; --------------------------------------------------------------------
	
play_note:
	dec  a
	add  a,a
	ld   e,a
	ld   d,0
	ld   hl,play_note_frequency_table
	add  hl,de
	ld   c,$a0
	sla  b
	inc  b
	out  (c),b
	inc  c
	outi
	dec  c
	out  (c),b
	inc  c
	outi
	ret

play_note_frequency_table:
        db     13,101, 12,157, 11,235, 11, 66, 10,154, 10,  4
        db      9,113,  8,232,  8,107,  7,242,  7,127,  7, 25
        db      6,179,  6, 78,  5,245,  5,161,  5, 77,  5,  2
        db      4,185,  4,116,  4, 52,  3,249,  3,192,  3,140
        db      3, 89,  3, 39,  2,251,  2,209,  2,167,  2,129
        db      2, 92,  2, 58,  2, 26,  1,253,  1,224,  1,198
        db      1,173,  1,148,  1,125,  1,104,  1, 83,  1, 65
        db      1, 46,  1, 29,  1, 13,  0,254,  0,240,  0,227
        db      0,214,  0,202,  0,191,  0,180,  0,170,  0,160
        db      0,151,  0,143,  0,135,  0,127,  0,120,  0,114
        db      0,107,  0,101,  0, 95,  0, 90,  0, 85,  0, 80
        db      0, 76,  0, 71,  0, 67,  0, 64,  0, 60,  0, 57
        db      0, 54,  0, 51,  0, 48,  0, 45,  0, 42,  0, 40
        db      0, 38,  0, 36,  0, 34,  0, 32,  0, 30,  0, 28
        db      0, 27,  0, 25,  0, 24,  0, 23,  0, 21,  0, 20
        db      0, 19,  0, 18,  0, 17,  0, 16,  0, 15,  0, 14
	
bin_end:	
